"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const path_1 = require("path");
const prompts_1 = require("@inquirer/prompts");
const package_json_1 = require("../package.json");
async function main() {
    const out = child_process_1.default.execSync(`git log v${package_json_1.version}..main --format="%s"`, {
        encoding: 'utf-8',
    });
    if (out) {
        const msgs = out.split('\n');
        let newVersion = package_json_1.version;
        const versionNum = package_json_1.version.split('.');
        if (msgs.some((each) => each.startsWith('feat:'))) {
            versionNum[1] = +versionNum[1] + 1;
        }
        else {
            versionNum[2] = +versionNum[2] + 1;
        }
        newVersion = await (0, prompts_1.input)({
            message: 'Please input the version you are going to release',
            default: versionNum.join('.'),
            validate: (val) => !!val,
        });
        console.log(`This is a ${newVersion} version`);
        await child_process_1.default.execFileSync('sh', [path_1.default.join(__dirname, './release.sh'), newVersion], {
            stdio: 'inherit',
        });
        let pemPath = '';
        const sendToServer = await (0, prompts_1.confirm)({
            message: 'Do you need to transfer the build file to server?',
            default: true,
        });
        if (sendToServer) {
            pemPath = await (0, prompts_1.input)({
                message: 'Please input server pem file path',
                default: `~/.ssh/web-server.pem`,
            });
            await child_process_1.default.execSync(`scp -i ${pemPath} -r dist-${newVersion}.tar.gz package.json ec2-user@54.213.114.202:/home/ec2-user/project/sea-admin-backend`, { stdio: 'inherit' });
        }
    }
    else {
        console.log('Nothing to release');
    }
}
main();
//# sourceMappingURL=release.js.map