"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseExceptionFilter = void 0;
const common_1 = require("@nestjs/common");
const dayjs = require("dayjs");
const nest_winston_1 = require("nest-winston");
const winston_1 = require("winston");
const response_constant_1 = require("../constant/response.constant");
const utils_1 = require("../util/utils");
let BaseExceptionFilter = exports.BaseExceptionFilter = class BaseExceptionFilter {
    constructor(logger) {
        this.logger = logger;
    }
    catch(exception, host) {
        const ctx = host.switchToHttp();
        const response = ctx.getResponse();
        const request = ctx.getRequest();
        let status = common_1.HttpStatus.INTERNAL_SERVER_ERROR;
        let message = '系统内部错误';
        let code = response_constant_1.RES_CODE.ERROR;
        let resData = exception === null || exception === void 0 ? void 0 : exception.message;
        if (exception instanceof common_1.HttpException) {
            status = exception.getStatus();
            const expectionResponse = exception.getResponse();
            if (typeof expectionResponse === 'string') {
                code = status;
                message = expectionResponse;
            }
            else {
                code = expectionResponse.code || code;
                message = expectionResponse.message || message;
                resData = expectionResponse.data || (exception === null || exception === void 0 ? void 0 : exception.message) || null;
            }
        }
        else if (request.error) {
            code = request.error.code;
            message = request.error.message;
        }
        else if ((exception === null || exception === void 0 ? void 0 : exception.code) === 'ENOENT') {
            status = exception.status;
            code = response_constant_1.RES_CODE.ERROR;
            message = '找不到资源';
        }
        const res = {
            code: code.toString(),
            timestamp: dayjs().format('YYYY-MM-DD HH:mm:ss'),
            path: request.url,
            message,
            resData,
        };
        this.logger.error(exception.stack, {
            request: (0, utils_1.getReqMeta)(request),
            response: res,
        });
        response.status(status).json(res);
    }
};
exports.BaseExceptionFilter = BaseExceptionFilter = __decorate([
    (0, common_1.Catch)(),
    __param(0, (0, common_1.Inject)(nest_winston_1.WINSTON_MODULE_PROVIDER)),
    __metadata("design:paramtypes", [winston_1.Logger])
], BaseExceptionFilter);
//# sourceMappingURL=base-exception.filter.js.map