/// <reference types="mongoose/types/aggregate" />
/// <reference types="mongoose/types/callback" />
/// <reference types="mongoose/types/collection" />
/// <reference types="mongoose/types/connection" />
/// <reference types="mongoose/types/cursor" />
/// <reference types="mongoose/types/document" />
/// <reference types="mongoose/types/error" />
/// <reference types="mongoose/types/expressions" />
/// <reference types="mongoose/types/helpers" />
/// <reference types="mongoose/types/middlewares" />
/// <reference types="mongoose/types/indexes" />
/// <reference types="mongoose/types/models" />
/// <reference types="mongoose/types/mongooseoptions" />
/// <reference types="mongoose/types/pipelinestage" />
/// <reference types="mongoose/types/populate" />
/// <reference types="mongoose/types/query" />
/// <reference types="mongoose/types/schemaoptions" />
/// <reference types="mongoose/types/schematypes" />
/// <reference types="mongoose/types/session" />
/// <reference types="mongoose/types/types" />
/// <reference types="mongoose/types/utility" />
/// <reference types="mongoose/types/validation" />
/// <reference types="mongoose/types/virtuals" />
/// <reference types="mongoose" />
/// <reference types="mongoose/types/inferschematype" />
import { ActivityEnv, ServerType } from '@/common/types/common.types';
import { UserInfo } from '@/common/types/sso.types';
import { ActivityConfigService } from './activity-config.service';
import { ActivityConfigType, ActivitySortConfig } from './common/types';
import { ActivityConfig } from './schemas/activity-config.schema';
export declare class ActivityConfigController {
    private readonly activityConfigService;
    constructor(activityConfigService: ActivityConfigService);
    syncData(user: UserInfo, env: ActivityEnv, port: number): Promise<{
        activity: import("mongodb").BulkWriteResult;
        wishGame: Promise<import("mongodb").BulkWriteResult>;
    }>;
    create(user: UserInfo, env: ActivityEnv, createActivityConfigDto: ActivityConfigType): Promise<(import("mongoose").Document<unknown, {}, import("mongoose").MergeType<import("mongoose").MergeType<ActivityConfig, {
        timeSt: Date;
        timeEnd: Date;
        env: ActivityEnv;
        creatorId: string;
        creatorName: string;
        updaterId: string;
        updaterName: string;
        port: number;
        serverType: string;
        transAc: 0 | 1;
        fixAc: 0 | 1;
        hottodayAc: 0 | 1;
        groups: string[];
        actRelated: string;
        useStoreInfo: boolean;
        priorityLoad: boolean;
        actCategory: string;
        indexOfAct: number;
        lobby: number;
        trans: number;
        fix: number;
        hottoday: number;
        _id: string;
        activityId: string;
        cateNum: number;
    }>, ActivityConfig & {
        _id: import("mongoose").Types.ObjectId;
    }>> & Omit<Omit<import("mongoose").MergeType<ActivityConfig, {
        timeSt: Date;
        timeEnd: Date;
        env: ActivityEnv;
        creatorId: string;
        creatorName: string;
        updaterId: string;
        updaterName: string;
        port: number;
        serverType: string;
        transAc: 0 | 1;
        fixAc: 0 | 1;
        hottodayAc: 0 | 1;
        groups: string[];
        actRelated: string;
        useStoreInfo: boolean;
        priorityLoad: boolean;
        actCategory: string;
        indexOfAct: number;
        lobby: number;
        trans: number;
        fix: number;
        hottoday: number;
        _id: string;
        activityId: string;
        cateNum: number;
    }>, keyof ActivityConfig> & ActivityConfig & {
        _id: import("mongoose").Types.ObjectId;
    }, never>)[]>;
    findAllInOneDay(env: ActivityEnv, serverType: ServerType, port: number | undefined, date: string): Promise<any[]>;
    update(user: UserInfo, id: string, activityConfig: ActivityConfigType): Promise<(import("mongoose").Document<unknown, {}, ActivityConfig> & Omit<ActivityConfig & {
        _id: import("mongoose").Types.ObjectId;
    }, never>) | null>;
    remove(user: UserInfo, id: string): Promise<(import("mongoose").Document<unknown, {}, ActivityConfig> & Omit<ActivityConfig & {
        _id: import("mongoose").Types.ObjectId;
    }, never>) | null>;
    clear(user: UserInfo, env: ActivityEnv, data: {
        serverType: string;
        port: number;
        date: string;
    }): Promise<{
        activity: import("mongoose").UpdateWriteOpResult;
        wishGame: import("mongoose").UpdateWriteOpResult | undefined;
    }>;
    getCountByDay(env: ActivityEnv, serverType: ServerType, port: number): Promise<{
        value: string;
        count: number;
    }[]>;
    sort(user: UserInfo, env: ActivityEnv, serverType: ServerType, port: number, date: string, activitySortConfigs: ActivitySortConfig[]): Promise<import("mongodb").BulkWriteResult>;
    getMultiOfTwoDays(env: ActivityEnv, serverType: ServerType, port: number, date: string, tgtEnv: ActivityEnv, tgtPort: number, tgtDate: string): Promise<any[]>;
    copy(user: UserInfo, env: ActivityEnv, serverType: ServerType, port: number, date: string, tgtEnv: ActivityEnv, tgtPort: number, tgtDate: string, extraConfigs: ActivityConfig[]): Promise<(import("mongoose").Document<unknown, {}, import("mongoose").MergeType<any, ActivityConfig & {
        _id: import("mongoose").Types.ObjectId;
    }>> & Omit<Omit<any, keyof ActivityConfig> & ActivityConfig & {
        _id: import("mongoose").Types.ObjectId;
    }, never>)[]>;
    getActiveUser(user: UserInfo, env: ActivityEnv, serverType: ServerType, port: string): Promise<{
        userId: string;
        username: string;
    }[]>;
    addActiveUser(user: UserInfo, env: ActivityEnv, serverType: ServerType, port: number): Promise<void>;
    delActiveUser(user: UserInfo, env: ActivityEnv, serverType: ServerType, port: number): Promise<void>;
    gitPush(env: ActivityEnv, port: number, serverType: ServerType): Promise<void>;
    gitPull(env: ActivityEnv, port: string): Promise<{
        date: string;
        activityConfigs: {
            name: string;
            url: string;
            port: number;
            env: string;
            serverType: string;
            activityId: string;
            timeSt: string;
            timeEnd: string;
            transAc: 0 | 1;
            fixAc: 0 | 1;
            hottodayAc: 0 | 1;
            groups: string[];
            actRelated: string;
            useStoreInfo: boolean;
            priorityLoad: boolean;
            actCategory: string;
            indexOfAct: number;
            lobby: number;
            trans: number;
            fix: number;
            hottoday: number;
            cateNum: number;
        }[];
        wishGameConfigs: {
            value: string;
            label: string;
        }[];
    }[]>;
    applyToProcess(): Promise<any>;
}
