"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActivityConfigService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("mongoose");
const dayjs = require("dayjs");
const mongoose_2 = require("@nestjs/mongoose");
const cache_manager_1 = require("@nestjs/cache-manager");
const mongo_1 = require("../../common/util/mongo");
const cache_constant_1 = require("../../common/constant/cache.constant");
const provider_1 = require("../../common/provider");
const activity_constant_1 = require("../../common/constant/activity.constant");
const activity_service_1 = require("../activity/activity.service");
const wish_game_config_service_1 = require("../wish-game-config/wish-game-config.service");
const activity_config_schema_1 = require("./schemas/activity-config.schema");
const utils_1 = require("./common/utils");
let ActivityConfigService = exports.ActivityConfigService = class ActivityConfigService {
    constructor(cacheManager, activityConfigModel, activityService, wishGameConfigService, activityEngineService) {
        this.cacheManager = cacheManager;
        this.activityConfigModel = activityConfigModel;
        this.activityService = activityService;
        this.wishGameConfigService = wishGameConfigService;
        this.activityEngineService = activityEngineService;
    }
    async syncData(user, env, port) {
        const engineRes = await this.activityEngineService.getDailyConfig();
        const { date_list } = engineRes;
        const activityConfigs = date_list.reduce((r, each) => [...r, ...each.activity], []);
        const nameToIdObj = await this.activityService.getIdObjByEnvAndName(env, activityConfigs.map((each) => each.name));
        const dataInDb = await this.activityConfigModel.find({
            env,
            port,
            timeSt: { $gte: (0, utils_1.getStandardDateOfActivity)().toDate() },
            timeEnd: {
                $lte: (0, utils_1.getStandardDateOfActivity)().add(16, 'day').toDate(),
            },
            isDeleted: false,
        });
        const dataInDbObj = {};
        dataInDb.forEach((each) => {
            dataInDbObj[each.activityId] = true;
        });
        const opts = [];
        activityConfigs.forEach((each) => {
            if (dataInDbObj[nameToIdObj[each.name]]) {
                opts.push({
                    updateOne: {
                        filter: { _id: new mongoose_1.Types.ObjectId(nameToIdObj[each.name]) },
                        update: Object.assign(Object.assign({}, (0, utils_1.getDocConfigFromEngineConfig)(each, env, port, nameToIdObj[each.name])), { updaterId: user.userid, updaterName: user.name }),
                    },
                });
            }
            else {
                opts.push({
                    insertOne: {
                        document: Object.assign(Object.assign({}, (0, utils_1.getDocConfigFromEngineConfig)(each, env, port, nameToIdObj[each.name])), { creatorId: user.userid, creatorName: user.name, updaterId: user.userid, updaterName: user.name }),
                    },
                });
            }
        });
        const activityRes = await this.activityConfigModel.bulkWrite(opts);
        const wishGameRes = this.wishGameConfigService.syncData(user, env, port, engineRes);
        return { activity: activityRes, wishGame: wishGameRes };
    }
    async create(user, env, activityConfig) {
        const dates = (0, utils_1.splitDate)(activityConfig.timeSt, activityConfig.timeEnd);
        const res = await this.activityConfigModel.insertMany(dates.map((date) => (Object.assign(Object.assign({}, activityConfig), { timeSt: date[0], timeEnd: date[1], env, creatorId: user.userid, creatorName: user.name, updaterId: user.userid, updaterName: user.name }))));
        await this.activityService.updateLastConfig(user, activityConfig.activityId, activityConfig);
        return res;
    }
    async findAllByDays(env, serverType, port, date, count = 1) {
        const filter = {
            env,
            serverType,
            port,
            timeSt: { $gte: (0, utils_1.getStandardDateOfActivity)(date).toDate() },
            timeEnd: {
                $lte: (0, utils_1.getStandardDateOfActivity)(date).add(count, 'day').toDate(),
            },
            isDeleted: false,
        };
        const res = await this.activityConfigModel.aggregate([
            { $match: filter },
            {
                $lookup: {
                    from: 'activities',
                    let: { activityId: '$activityId' },
                    pipeline: [
                        {
                            $addFields: { _idString: { $toString: '$_id' } },
                        },
                        { $match: { $expr: { $eq: ['$_idString', '$$activityId'] } } },
                    ],
                    as: 'activity',
                },
            },
            {
                $addFields: {
                    name: { $first: '$activity.name' },
                    url: { $first: '$activity.url' },
                    cateNum: { $first: '$activity.cateNum' },
                    timeSt: { $dateToString: (0, mongo_1.default)('$timeSt') },
                    timeEnd: { $dateToString: (0, mongo_1.default)('$timeEnd') },
                    createdAt: { $dateToString: (0, mongo_1.default)('$createdAt') },
                    updatedAt: { $dateToString: (0, mongo_1.default)('$updatedAt') },
                },
            },
            { $project: { activity: 0 } },
        ]);
        return res;
    }
    async update(user, id, activityConfig) {
        const res = await this.activityConfigModel.findOneAndUpdate({
            _id: id,
            isDeleted: false,
        }, Object.assign(Object.assign({}, activityConfig), { updaterId: user.userid, updaterName: user.name }));
        await this.activityService.updateLastConfig(user, activityConfig.activityId, activityConfig);
        return res;
    }
    async remove(user, id) {
        const res = await this.activityConfigModel.findOneAndUpdate({
            _id: id,
            isDeleted: false,
        }, {
            isDeleted: true,
            updaterId: user.userid,
            updaterName: user.name,
        });
        return res;
    }
    async clear(user, env, serverType, port, date) {
        const aRes = await this.activityConfigModel.updateMany({
            env,
            serverType,
            port,
            timeSt: { $gte: (0, utils_1.getStandardDateOfActivity)(date).toDate() },
            timeEnd: {
                $lte: (0, utils_1.getStandardDateOfActivity)(date).add(1, 'day').toDate(),
            },
            isDeleted: false,
        }, {
            isDeleted: true,
            updaterId: user.userid,
            updaterName: user.name,
        });
        let wRes;
        if (serverType === activity_constant_1.SERVER_TYPE.DAFU) {
            wRes = await this.wishGameConfigService.clear(user, env, port, date);
        }
        return {
            activity: aRes,
            wishGame: wRes,
        };
    }
    async getCountByDay(env, serverType, port) {
        const res = [];
        for (let i = 0; i < 16; i += 1) {
            const date = (0, utils_1.getStandardDateOfActivity)((0, utils_1.getNow)().add(i, 'day'));
            const count = await this.activityConfigModel
                .find({
                env,
                serverType,
                port,
                timeSt: { $gte: date.toDate() },
                timeEnd: { $lte: date.add(1, 'day').toDate() },
                isDeleted: false,
            })
                .countDocuments();
            res.push({
                value: date.format('YYYY-MM-DD'),
                count,
            });
        }
        return res;
    }
    async sort(user, env, serverType, port, date, activitySortConfigs) {
        const opts = activitySortConfigs.map((activitySortConfig) => ({
            updateOne: {
                filter: {
                    env,
                    serverType,
                    port,
                    timeSt: { $gte: (0, utils_1.getStandardDateOfActivity)(date).toDate() },
                    timeEnd: {
                        $lte: (0, utils_1.getStandardDateOfActivity)(date).add(1, 'day').toDate(),
                    },
                    _id: activitySortConfig._id,
                },
                update: {
                    indexOfAct: activitySortConfig.indexOfAct,
                    lobby: activitySortConfig.lobby,
                    trans: activitySortConfig.trans,
                    fix: activitySortConfig.fix,
                    hottoday: activitySortConfig.hottoday,
                    updaterId: user.userid,
                    updaterName: user.name,
                },
            },
        }));
        const res = this.activityConfigModel.bulkWrite(opts);
        return res;
    }
    async getMultiOfOneDay(env, serverType, port, date) {
        const pre = await this.findAllByDays(env, serverType, port, dayjs(date).subtract(1, 'day').format('YYYY-MM-DD'));
        const current = await this.findAllByDays(env, serverType, port, date);
        const next = await this.findAllByDays(env, serverType, port, dayjs(date).add(1, 'day').format('YYYY-MM-DD'));
        const siblings = [...pre, ...next];
        const res = current.filter((config) => siblings.find((each) => each.activityId === config.activityId));
        return res;
    }
    async getMultiOfTwoDays(env, serverType, port, date, tgtEnv, tgtPort, tgtDate) {
        const currentConfigs = await this.findAllByDays(env, serverType, port, date);
        const tgtMultiConfigs = await this.getMultiOfOneDay(tgtEnv, serverType, tgtPort, tgtDate);
        const res = tgtMultiConfigs.filter((each) => !currentConfigs.find((config) => config.activityId === each.activityId));
        return res;
    }
    async copy(user, env, serverType, port, date, tgtEnv, tgtPort, tgtDate, extraConfigs) {
        const baseActivityConfigs = await this.findAllByDays(env, serverType, port, date);
        await this.clear(user, tgtEnv, serverType, tgtPort, tgtDate);
        const activityMeta = await this.activityService.getActivityIdsByEnv(env, tgtEnv, baseActivityConfigs.map((each) => each.activityId));
        const diffDays = dayjs(tgtDate).diff(date, 'day');
        const baseRes = baseActivityConfigs.map((each) => (Object.assign(Object.assign({}, each), { _id: new mongoose_1.Types.ObjectId(), activityId: activityMeta[each.activityId], timeSt: dayjs(each.timeSt).add(diffDays, 'day').toDate(), timeEnd: dayjs(each.timeEnd).add(diffDays, 'day').toDate(), env: tgtEnv, serverType, port: tgtPort, creatorId: user.userid, creatorName: user.name, updaterId: user.userid, updaterName: user.name })));
        const extraRes = extraConfigs.map((each) => (Object.assign(Object.assign({}, each), { _id: new mongoose_1.Types.ObjectId(), env: tgtEnv, serverType, port: tgtPort, creatorId: user.userid, creatorName: user.name, updaterId: user.userid, updaterName: user.name })));
        const res = await this.activityConfigModel.insertMany([
            ...baseRes,
            ...extraRes,
        ]);
        return res;
    }
    getCacheKey(env, serverType, port) {
        const envs = {
            'dev-gtm': cache_constant_1.DEV_GTM_ACTIVE_USER_LIST,
            'dev-rtm': cache_constant_1.DEV_RTM_ACTIVE_USER_LIST,
            'prod-rtm': cache_constant_1.PROD_RTM_ACTIVE_USER_LIST,
            'prod-online': cache_constant_1.PROD_ONLINE_ACTIVE_USER_LIST,
        };
        return `${envs[env]}-${serverType}-${port}`;
    }
    async getActiveUser(user, env, serverType, port) {
        const cacheKey = this.getCacheKey(env, serverType, port);
        const data = ((await this.cacheManager.get(cacheKey)) || []);
        const index = data.findIndex((each) => each.userId === user.userid);
        return index === -1 ? data : data.slice(0, index);
    }
    async addActiveUser(user, env, serverType, port) {
        const cacheKey = this.getCacheKey(env, serverType, port);
        let data = ((await this.cacheManager.get(cacheKey)) || []);
        data = [
            ...data.filter((each) => each.userId !== user.userid),
            { userId: user.userid, username: user.name },
        ];
        await this.cacheManager.set(cacheKey, data);
        return;
    }
    async delActiveUser(user, env, serverType, port) {
        const cacheKey = this.getCacheKey(env, serverType, port);
        let data = ((await this.cacheManager.get(cacheKey)) || []);
        data = data.filter((each) => each.userId !== user.userid);
        if (data.length === 0) {
            await this.cacheManager.del(cacheKey);
        }
        else {
            await this.cacheManager.set(cacheKey, data);
        }
        return;
    }
    async gitPush(env, serverType, port) {
        const activitiesInDb = await this.findAllByDays(env, serverType, port, (0, utils_1.getNow)().format('YYYY-MM-DD'), 16);
        const wishGamesInDb = await this.wishGameConfigService.findAllByDays(env, port, (0, utils_1.getDatesStrList)(16));
        const data = (0, utils_1.splitConfigsByDate)(activitiesInDb, wishGamesInDb);
        const dataInEngine = await this.activityEngineService.getDailyConfig();
        data.map((each, index) => {
            each.activity.push(...dataInEngine.date_list[index].activity.filter((each) => serverType === activity_constant_1.SERVER_TYPE.DAFU
                ? each.cate_num > 9
                : each.cate_num <= 9));
        });
        await this.activityEngineService.gitPush(data);
        return;
    }
    async gitPull(env, port) {
        console.log(env, port);
        const data = await this.activityEngineService.getDailyConfig();
        return (0, utils_1.formatEngineConfigs)(data);
    }
    async applyToProcess() {
        return await this.activityEngineService.applyToProcess();
    }
};
exports.ActivityConfigService = ActivityConfigService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(cache_manager_1.CACHE_MANAGER)),
    __param(1, (0, mongoose_2.InjectModel)(activity_config_schema_1.ActivityConfig.name)),
    __metadata("design:paramtypes", [Object, mongoose_1.Model,
        activity_service_1.ActivityService,
        wish_game_config_service_1.WishGameConfigService,
        provider_1.ActivityEngineSerivce])
], ActivityConfigService);
//# sourceMappingURL=activity-config.service.js.map