"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatEngineConfigs = exports.splitConfigsByDate = exports.getDateStrByTimeSt = exports.getDatesStrList = exports.getEngineConfigFromDocConfig = exports.getDocConfigFromEngineConfig = exports.getStandardDateOfActivity = exports.splitDate = exports.getNow = void 0;
const dayjs = require("dayjs");
const activity_constant_1 = require("../../../common/constant/activity.constant");
function getNow() {
    return dayjs().subtract(11, 'hour');
}
exports.getNow = getNow;
function splitDate(start, end) {
    const startDate = dayjs(start);
    const startHour = startDate.hour();
    const endDate = dayjs(end);
    const res = [];
    let nextStart = startDate;
    let nextEnd = startDate
        .clone()
        .add(startHour < 11 ? 0 : 1, 'day')
        .set('hour', 11)
        .set('minute', 0)
        .set('second', 0);
    while (nextEnd.isBefore(endDate)) {
        res.push([nextStart.toDate(), nextEnd.toDate()]);
        nextStart = nextEnd;
        nextEnd = nextEnd.add(1, 'day');
    }
    res.push([nextStart.toDate(), endDate.toDate()]);
    return res;
}
exports.splitDate = splitDate;
function getStandardDateOfActivity(date) {
    return dayjs(date)
        .clone()
        .set('hour', 11)
        .set('minute', 0)
        .set('second', 0)
        .set('millisecond', 0);
}
exports.getStandardDateOfActivity = getStandardDateOfActivity;
function getDocConfigFromEngineConfig(config, env, port, activityId) {
    const { act_category, act_related, cate_num, dab_type_1, dab_type_2, fix, fix_ac, hottoday, hottoday_ac, index_of_act, lobby, priority_load, time_end, time_st, trans, trans_ac, use_store_info, user_type_A, user_type_B, } = config;
    const groups = [];
    if (user_type_A === 1) {
        if (dab_type_1 === 1) {
            groups.push('A1');
        }
        if (dab_type_2 === 1) {
            groups.push('A2');
        }
    }
    if (user_type_B === 1) {
        if (dab_type_1 === 1) {
            groups.push('B1');
        }
        if (dab_type_2 === 1) {
            groups.push('B2');
        }
    }
    return {
        env,
        serverType: cate_num > 9 ? activity_constant_1.SERVER_TYPE.SAIL : activity_constant_1.SERVER_TYPE.DAFU,
        port,
        activityId,
        timeSt: time_st,
        timeEnd: time_end,
        transAc: trans_ac,
        fixAc: fix_ac,
        hottodayAc: hottoday_ac,
        groups,
        actRelated: act_related,
        useStoreInfo: use_store_info,
        priorityLoad: priority_load,
        actCategory: act_category,
        cateNum: cate_num,
        indexOfAct: index_of_act,
        lobby,
        trans,
        fix,
        hottoday,
    };
}
exports.getDocConfigFromEngineConfig = getDocConfigFromEngineConfig;
function getEngineConfigFromDocConfig(config) {
    const { name, url, timeSt, timeEnd, transAc, fixAc, hottodayAc, groups, actRelated, useStoreInfo, priorityLoad, actCategory, cateNum, indexOfAct, lobby, trans, fix, hottoday, } = config;
    return {
        name,
        url,
        act_category: actCategory,
        act_related: actRelated,
        cate_num: cateNum,
        dab_type_1: groups.some((group) => group.includes('1')) ? 1 : 0,
        dab_type_2: groups.some((group) => group.includes('2')) ? 1 : 0,
        fix,
        fix_ac: fixAc,
        hottoday,
        hottoday_ac: hottodayAc,
        index_of_act: indexOfAct,
        lobby,
        priority_load: priorityLoad,
        time_end: timeEnd,
        time_st: timeSt,
        trans,
        trans_ac: transAc,
        use_store_info: useStoreInfo,
        user_type_A: groups.some((group) => group.includes('A')) ? 1 : 0,
        user_type_B: groups.some((group) => group.includes('B')) ? 1 : 0,
    };
}
exports.getEngineConfigFromDocConfig = getEngineConfigFromDocConfig;
function getDatesStrList(count) {
    const dates = [];
    for (let i = 0; i < count; i += 1) {
        dates.push(getNow().add(i, 'day').format('YYYY-MM-DD'));
    }
    return dates;
}
exports.getDatesStrList = getDatesStrList;
function getDateStrByTimeSt(date) {
    return dayjs(date).subtract(11, 'hour').format('YYYY-MM-DD');
}
exports.getDateStrByTimeSt = getDateStrByTimeSt;
function splitConfigsByDate(activityConfigsInDb, wishGameConfigs) {
    const res = {};
    for (let i = 0; i < 16; i += 1) {
        const date = getNow().add(i, 'day').format('YYYY-MM-DD');
        res[date] = {
            date,
            wish_game: [],
            activity: [],
            open_card_group: [],
        };
    }
    activityConfigsInDb.forEach((config) => {
        const date = getDateStrByTimeSt(config.timeSt);
        if (res[date]) {
            res[date].activity.push(getEngineConfigFromDocConfig(config));
        }
    });
    wishGameConfigs.forEach((config) => {
        if (res[config.date]) {
            res[config.date].wish_game.push({
                game_name: config.value,
            });
        }
    });
    return Object.values(res);
}
exports.splitConfigsByDate = splitConfigsByDate;
function formatEngineConfigs(data) {
    const wishGameObj = {};
    data.wish_themes.forEach((each) => {
        wishGameObj[each.id] = each.name;
    });
    return data.date_list.map((each) => {
        return {
            date: each.date,
            activityConfigs: each.activity.map((e) => (Object.assign(Object.assign({}, getDocConfigFromEngineConfig(e, activity_constant_1.ACTIVITY_ENV.DEV_GTM, -1, '')), { name: e.name, url: e.url }))),
            wishGameConfigs: each.wish_game.map((e) => ({
                value: e.game_name,
                label: wishGameObj[e.game_name],
            })),
        };
    });
}
exports.formatEngineConfigs = formatEngineConfigs;
//# sourceMappingURL=utils.js.map