"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActivityService = void 0;
const mongoose_1 = require("mongoose");
const common_1 = require("@nestjs/common");
const mongoose_2 = require("@nestjs/mongoose");
const mongo_1 = require("../../common/util/mongo");
const response_constant_1 = require("../../common/constant/response.constant");
const provider_1 = require("../../common/provider");
const activity_constant_1 = require("../../common/constant/activity.constant");
const activity_schema_1 = require("./schemas/activity.schema");
let ActivityService = exports.ActivityService = class ActivityService {
    constructor(activityModel, activityEngineService) {
        this.activityModel = activityModel;
        this.activityEngineService = activityEngineService;
    }
    async syncData(user, env) {
        const { activities } = await this.activityEngineService.getDailyConfig();
        const activityList = activities.reduce((r, each) => [...r, ...each.data], []);
        const dataInDb = await this.activityModel.find({
            env,
        });
        const dataInDbObj = {};
        dataInDb.forEach((each) => {
            dataInDbObj[each.name] = each._id;
        });
        const opts = [];
        activityList.forEach((each) => {
            if (dataInDbObj[each.name]) {
                opts.push({
                    updateOne: {
                        filter: { _id: new mongoose_1.Types.ObjectId(dataInDbObj[each.name]) },
                        update: {
                            name: each.name,
                            url: each.url,
                            cateNum: each.recent_cate_num,
                            env,
                            updaterId: user.userid,
                            updaterName: user.name,
                        },
                    },
                });
            }
            else {
                opts.push({
                    insertOne: {
                        document: {
                            name: each.name,
                            url: each.url,
                            cateNum: each.recent_cate_num,
                            env,
                            creatorId: user.userid,
                            creatorName: user.name,
                            updaterId: user.userid,
                            updaterName: user.name,
                        },
                    },
                });
            }
        });
        const res = await this.activityModel.bulkWrite(opts);
        return res;
    }
    async findAllInOnePage(env, page, pageSize, searchValue, cateNum, serverType) {
        const searchValueReg = new RegExp(searchValue);
        const filter = { env };
        if (typeof cateNum !== 'undefined') {
            filter.cateNum = cateNum;
        }
        else if (typeof serverType !== 'undefined') {
            filter.cateNum = {
                $in: serverType === activity_constant_1.SERVER_TYPE.DAFU
                    ? [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
                    : [0, 10, 11, 12, 13, 14, 15, 16],
            };
        }
        if (searchValue) {
            filter.name = searchValueReg;
        }
        const total = await this.activityModel.find(filter).countDocuments();
        const data = await this.activityModel.aggregate([
            { $match: filter },
            { $sort: { updatedAt: -1 } },
            { $skip: Math.max(page - 1, 0) * pageSize },
            { $limit: pageSize },
            {
                $addFields: {
                    createdAt: { $dateToString: (0, mongo_1.default)('$createdAt') },
                    updatedAt: { $dateToString: (0, mongo_1.default)('$updatedAt') },
                },
            },
        ]);
        return { total, data };
    }
    async updateCateNum(user, data) {
        const { ids, cateNum } = data;
        const res = await this.activityModel.updateMany({ _id: { $in: ids } }, {
            cateNum,
            updaterId: user.userid,
            updaterName: user.name,
        });
        return res;
    }
    async updateLastConfig(user, id, data) {
        const res = await this.activityModel.updateMany({ _id: id }, {
            cateNum: data.cateNum,
            lastConfig: Object.assign(Object.assign({}, data), { updaterId: user.userid, updaterName: user.name, updatedAt: new Date() }),
        });
        return res;
    }
    async getActivityIdsByEnv(srcEnv, tgtEnv, ids) {
        const srcRes = await this.activityModel
            .find({
            env: srcEnv,
            _id: {
                $in: ids,
            },
        }, { _id: 1, name: 1 })
            .lean();
        const tgtRes = await this.activityModel
            .find({
            env: tgtEnv,
            name: {
                $in: srcRes.map((each) => each.name),
            },
        }, { _id: 1, name: 1 })
            .lean();
        if (srcRes.length !== tgtRes.length) {
            throw new common_1.HttpException({
                code: response_constant_1.RES_CODE.CREATE_ERROR,
                message: '复制失败：来源环境和目标环境活动数据不一致',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        const res = {};
        srcRes.map((each) => {
            var _a;
            res[each._id] = (_a = tgtRes.find((e) => e.name === each.name)) === null || _a === void 0 ? void 0 : _a._id;
        });
        return res;
    }
    async getIdObjByEnvAndName(env, names) {
        const tgtRes = await this.activityModel
            .find({
            env: env,
            name: {
                $in: names,
            },
        }, { _id: 1, name: 1 })
            .lean();
        const res = {};
        names.forEach((name) => {
            const tgt = tgtRes.find((each) => each.name === name);
            if (tgt) {
                res[name] = tgt._id.toString();
            }
            else {
                throw new common_1.HttpException({
                    code: response_constant_1.RES_CODE.CREATE_ERROR,
                    message: '同步失败：当前环境和远端环境活动数据不一致',
                }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
            }
        });
        return res;
    }
};
exports.ActivityService = ActivityService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_2.InjectModel)(activity_schema_1.Activity.name)),
    __metadata("design:paramtypes", [mongoose_1.Model,
        provider_1.ActivityEngineSerivce])
], ActivityService);
//# sourceMappingURL=activity.service.js.map