"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CopywritingCategoryService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("mongoose");
const mongoose_2 = require("@nestjs/mongoose");
const cache_manager_1 = require("@nestjs/cache-manager");
const response_constant_1 = require("../../common/constant/response.constant");
const cache_constant_1 = require("../../common/constant/cache.constant");
const copywriting_service_1 = require("../copywriting/copywriting.service");
const constants_1 = require("./common/constants");
let CopywritingCategoryService = exports.CopywritingCategoryService = class CopywritingCategoryService {
    constructor(cacheManager, copywritingCategoryModel, copywritingService) {
        this.cacheManager = cacheManager;
        this.copywritingCategoryModel = copywritingCategoryModel;
        this.copywritingService = copywritingService;
    }
    async init(user) {
        const total = await this.copywritingCategoryModel
            .find({ isDeleted: false })
            .count();
        if (total > 0) {
            throw new common_1.HttpException({
                code: response_constant_1.RES_CODE.CREATE_ERROR,
                message: '分类初始化失败',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        const res = await this.copywritingCategoryModel.insertMany(constants_1.INIT_CATE.map((each, index) => (Object.assign(Object.assign({}, each), { order: index, creatorId: user.userid, creatorName: user.name }))));
        await this.setAndGetAllInCacheFromDb();
        return res;
    }
    async create(user, copywritingDto) {
        await this.checkRepeated(new mongoose_1.Types.ObjectId(), copywritingDto.label);
        const total = await this.copywritingCategoryModel.find().count();
        const { _id } = copywritingDto, rest = __rest(copywritingDto, ["_id"]);
        const res = await this.copywritingCategoryModel.create(Object.assign(Object.assign({}, rest), { order: total, creatorId: user.userid, creatorName: user.name }));
        await this.setAndGetAllInCacheFromDb();
        return res;
    }
    async setAndGetAllInCacheFromDb() {
        const res = await this.copywritingCategoryModel
            .find({ isDeleted: false })
            .sort({ order: 1 })
            .lean();
        await this.cacheManager.set(cache_constant_1.CW_CATE_LIST, res);
        return res;
    }
    async findAll() {
        let res = await this.cacheManager.get(cache_constant_1.CW_CATE_LIST);
        if (!res) {
            res = await this.setAndGetAllInCacheFromDb();
        }
        return res;
    }
    async findOne(id) {
        const res = await this.copywritingCategoryModel.findOne({
            isDeleted: false,
            _id: id,
        });
        return res;
    }
    async findOneByLabel(label) {
        const res = await this.copywritingCategoryModel.findOne({
            isDeleted: false,
            label,
        });
        return res;
    }
    async update(id, copywritingDto) {
        await this.checkRepeated(id, copywritingDto.label);
        const res = await this.copywritingCategoryModel.updateOne({ _id: id }, copywritingDto);
        await this.setAndGetAllInCacheFromDb();
        return res;
    }
    async remove(id) {
        const ids = await this.findSelfAndChildrenIdsById(id);
        const res = await this.copywritingCategoryModel.updateMany({ _id: { $in: ids } }, { isDeleted: true });
        await this.copywritingService.batchRemoveByCategoryId(ids);
        await this.setAndGetAllInCacheFromDb();
        return res;
    }
    async sort(updateCopywritingDtos) {
        const ops = updateCopywritingDtos.map((each, index) => ({
            updateOne: {
                filter: { _id: each._id },
                update: Object.assign(Object.assign({}, each), { order: index }),
            },
        }));
        const res = await this.copywritingCategoryModel.bulkWrite(ops);
        await this.setAndGetAllInCacheFromDb();
        return res;
    }
    async checkRepeated(id, label) {
        const repeatedModel = await this.copywritingCategoryModel.findOne({
            label,
            isDeleted: false,
            _id: {
                $ne: id,
            },
        });
        if (repeatedModel) {
            throw new common_1.HttpException({
                code: response_constant_1.RES_CODE.VALIDATE_ERROR,
                message: '已存在相同名称，请重新命名',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        return;
    }
    async findSelfAndChildrenIdsById(id) {
        const list = await this.findAll();
        let res = [];
        if (!id) {
            res = list.map((each) => each._id);
        }
        else {
            res = [id];
            let lastLevel = [id];
            while (lastLevel.length > 0) {
                const thisLevel = [];
                list.forEach((each) => {
                    if (lastLevel.includes(each.parentId)) {
                        thisLevel.push(each._id);
                    }
                });
                res.push(...thisLevel);
                lastLevel = thisLevel;
            }
        }
        return res;
    }
};
exports.CopywritingCategoryService = CopywritingCategoryService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(cache_manager_1.CACHE_MANAGER)),
    __param(1, (0, mongoose_2.InjectModel)('CopywritingCategory')),
    __param(2, (0, common_1.Inject)((0, common_1.forwardRef)(() => copywriting_service_1.CopywritingService))),
    __metadata("design:paramtypes", [Object, mongoose_1.Model,
        copywriting_service_1.CopywritingService])
], CopywritingCategoryService);
//# sourceMappingURL=copywriting-category.service.js.map