"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CopywritingErrorService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const response_constant_1 = require("../../common/constant/response.constant");
const mongo_1 = require("../../common/util/mongo");
const constants_1 = require("./common/constants");
const copywriting_error_schema_1 = require("./schemas/copywriting-error.schema");
let CopywritingErrorService = exports.CopywritingErrorService = class CopywritingErrorService {
    constructor(copywritingErrorModel) {
        this.copywritingErrorModel = copywritingErrorModel;
    }
    async create(user, copywritingErrorDto) {
        const sameOne = await this.findSameOne('', copywritingErrorDto.cn, copywritingErrorDto.tw, copywritingErrorDto.en);
        if (sameOne) {
            throw new common_1.HttpException({
                code: response_constant_1.RES_CODE.VALIDATE_ERROR,
                message: '易错词条已存在',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        const fullData = Object.assign(Object.assign({}, copywritingErrorDto), { creatorId: user.userid, creatorName: user.name, history: [
                {
                    type: constants_1.OPER_TYPE_ENUM.ADD,
                    operator: user.name,
                    createdAt: Date.now(),
                    record: copywritingErrorDto,
                },
            ] });
        const res = await this.copywritingErrorModel.create(fullData);
        return res;
    }
    async findAll() {
        const data = await this.copywritingErrorModel
            .find({
            isDeleted: false,
        })
            .lean();
        return data;
    }
    async findAllInOnePage(page, pageSize, searchText, createTime) {
        const searchTextReg = new RegExp(searchText);
        const [start, end] = createTime === null || createTime === void 0 ? void 0 : createTime.split(',');
        const createdAtFilter = end
            ? {
                createdAt: {
                    $gte: new Date(start),
                    $lte: new Date(end),
                },
            }
            : {};
        const filter = Object.assign(Object.assign({ isDeleted: false }, createdAtFilter), { $or: [
                { name: searchTextReg },
                { cn: searchTextReg },
                { tw: searchTextReg },
                { en: searchTextReg },
            ] });
        const total = await this.copywritingErrorModel
            .find(filter)
            .countDocuments();
        const data = await this.copywritingErrorModel.aggregate([
            { $match: filter },
            { $sort: { updatedAt: -1 } },
            { $skip: Math.max(page - 1, 0) * pageSize },
            { $limit: pageSize },
            {
                $addFields: {
                    createdAt: { $dateToString: (0, mongo_1.default)('$createdAt') },
                    updatedAt: { $dateToString: (0, mongo_1.default)('$updatedAt') },
                },
            },
        ]);
        return { data, total };
    }
    async findOne(id) {
        const res = (await this.copywritingErrorModel.aggregate([
            { $match: { isDeleted: false, _id: new mongoose_2.Types.ObjectId(id) } },
            { $limit: 1 },
            {
                $addFields: {
                    createdAt: { $dateToString: (0, mongo_1.default)('$createdAt') },
                    updatedAt: { $dateToString: (0, mongo_1.default)('$updatedAt') },
                    history: {
                        createdAt: { $dateToString: (0, mongo_1.default)('$createdAt') },
                    },
                },
            },
        ]))[0];
        return res;
    }
    async update(user, id, copywritingErrorDto) {
        let res = null;
        const sameOne = await this.findSameOne(id, copywritingErrorDto.cn, copywritingErrorDto.tw, copywritingErrorDto.en);
        if (sameOne) {
            throw new common_1.HttpException({
                code: response_constant_1.RES_CODE.VALIDATE_ERROR,
                message: '易错词条已存在',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        const target = await this.findOne(id);
        if (target) {
            const newDto = Object.assign(Object.assign({}, target), copywritingErrorDto);
            const type = this.getType(target, newDto);
            let newHistory = target.history;
            if (type) {
                newHistory = [
                    {
                        type,
                        operator: user.name,
                        createdAt: Date.now(),
                        record: newDto,
                    },
                    ...target.history,
                ];
            }
            res = await this.copywritingErrorModel.updateOne({ _id: id }, Object.assign(Object.assign({}, copywritingErrorDto), { history: newHistory }));
        }
        return res;
    }
    async remove(id) {
        const res = await this.copywritingErrorModel.updateOne({ _id: id }, { isDeleted: true });
        return res;
    }
    async findHistory(id) {
        const res = await this.findOne(id);
        return (res === null || res === void 0 ? void 0 : res.history) || [];
    }
    getType(oldRecord, newRecord) {
        const resTypes = [];
        if (oldRecord.cn !== newRecord.cn) {
            resTypes.push(constants_1.OPER_TYPE_ENUM.CN);
        }
        if (oldRecord.tw !== newRecord.tw) {
            resTypes.push(constants_1.OPER_TYPE_ENUM.TW);
        }
        if (oldRecord.en !== newRecord.en) {
            resTypes.push(constants_1.OPER_TYPE_ENUM.EN);
        }
        if (oldRecord.name !== newRecord.name ||
            oldRecord.desc !== newRecord.desc) {
            resTypes.push(constants_1.OPER_TYPE_ENUM.COMMON);
        }
        return resTypes.join(',');
    }
    async findSameOne(id, cn, tw, en) {
        const idFilter = id ? { _id: { $ne: id } } : {};
        const orFilter = [];
        if (cn) {
            orFilter.push({ cn });
        }
        if (tw) {
            orFilter.push({ tw });
        }
        if (en) {
            orFilter.push({ en });
        }
        if (orFilter.length === 0) {
            return null;
        }
        const tgt = await this.copywritingErrorModel.findOne(Object.assign(Object.assign({}, idFilter), { isDeleted: false, $or: orFilter }));
        return tgt;
    }
    async validateContent(key, value, id = '') {
        const idFilter = id ? { _id: { $ne: id } } : {};
        const orFilter = [];
        if (value) {
            orFilter.push({ [key]: value });
        }
        if (Object.keys(idFilter).length === 0 && orFilter.length === 0) {
            return null;
        }
        const tgt = await this.copywritingErrorModel.findOne(Object.assign(Object.assign({}, idFilter), { isDeleted: false, $or: orFilter }));
        return !tgt;
    }
};
exports.CopywritingErrorService = CopywritingErrorService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)(copywriting_error_schema_1.CopywritingError.name)),
    __metadata("design:paramtypes", [mongoose_2.Model])
], CopywritingErrorService);
//# sourceMappingURL=copywriting-error.service.js.map