"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CopywritingService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const node_xlsx_1 = require("node-xlsx");
const utils_1 = require("../../common/util/utils");
const response_constant_1 = require("../../common/constant/response.constant");
const auth_constant_1 = require("../../common/constant/auth.constant");
const copywriting_category_service_1 = require("../copywriting-category/copywriting-category.service");
const constants_1 = require("./common/constants");
let CopywritingService = exports.CopywritingService = class CopywritingService {
    constructor(copywritingModel, copywritingCategoryService) {
        this.copywritingModel = copywritingModel;
        this.copywritingCategoryService = copywritingCategoryService;
    }
    async create(user, copywritingDto) {
        const currentCategory = await this.copywritingCategoryService.findOne(copywritingDto.categoryId);
        const sameOne = await this.findSameCopywriting('', copywritingDto.cn, copywritingDto.tw, copywritingDto.en);
        if (sameOne) {
            throw new common_1.HttpException({
                code: response_constant_1.RES_CODE.VALIDATE_ERROR,
                message: '文案内容已存在',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        const fullData = Object.assign(Object.assign({}, copywritingDto), { creatorId: user.userid, creatorName: user.name, history: [
                {
                    type: constants_1.OPER_TYPE_ENUM.ADD,
                    operator: user.name,
                    createdAt: Date.now(),
                    record: Object.assign(Object.assign({}, copywritingDto), { categoryName: currentCategory === null || currentCategory === void 0 ? void 0 : currentCategory.label }),
                },
            ] });
        const res = await this.copywritingModel.create(fullData);
        return res;
    }
    async batchCreateFromFile(user, file) {
        const workSheets = node_xlsx_1.default.parse(file.buffer);
        const data = [];
        const errorList = [];
        const categoryList = await this.copywritingCategoryService.findAll();
        workSheets.forEach((workSheet) => {
            workSheet.data.map((each, index) => {
                if (index !== 0) {
                    data.push({
                        name: each[0],
                        categoryName: each[1],
                        cn: each[2],
                        tw: each[3],
                        en: each[4],
                        desc: each[5],
                    });
                }
            });
        });
        for (const each of data) {
            if (each.name && each.categoryName) {
                try {
                    const currentCategory = categoryList.find((category) => category.label === each.categoryName);
                    if (currentCategory) {
                        const terminalCategory = (0, utils_1.findTerminalNodeInForestList)(categoryList, currentCategory._id.toString(), 'root', '_id', 'parentId');
                        const isAuthed = terminalCategory &&
                            user.permissions.find((permission) => constants_1.MODULE_TO_PERMISSION[terminalCategory.label].content === permission[2] && permission[3] === auth_constant_1.AUTH.CREATE);
                        if (isAuthed) {
                            const dto = Object.assign(Object.assign({}, each), { categoryId: currentCategory === null || currentCategory === void 0 ? void 0 : currentCategory._id });
                            const sameOne = await this.findSameCopywriting('', each.cn, each.tw, each.en);
                            if (!sameOne) {
                                await this.copywritingModel.create(Object.assign(Object.assign({}, dto), { creatorId: user.userid, creatorName: user.name, history: [
                                        {
                                            type: constants_1.OPER_TYPE_ENUM.ADD,
                                            operator: user.name,
                                            record: Object.assign(Object.assign({}, dto), { categoryName: each.categoryName }),
                                        },
                                    ] }));
                            }
                            else {
                                errorList.push({
                                    name: each.name,
                                    msg: '文案内容已存在',
                                });
                            }
                        }
                        else {
                            errorList.push({
                                name: each.name,
                                msg: `您暂无权限在该分类【${each.categoryName}】中创建文案`,
                            });
                        }
                    }
                    else {
                        errorList.push({
                            name: each.name,
                            msg: '文案分类不存在',
                        });
                    }
                }
                catch (err) {
                    errorList.push({
                        name: each.name,
                        msg: '文案创建失败',
                    });
                }
            }
            else {
                errorList.push({
                    name: each.name,
                    msg: '文案名称、分类不能为空',
                });
            }
        }
        return errorList;
    }
    async findAll(categoryId, page, pageSize, searchText, createTime) {
        const searchTextReg = new RegExp(searchText);
        const [start, end] = createTime === null || createTime === void 0 ? void 0 : createTime.split(',');
        const categoryIds = await this.copywritingCategoryService.findSelfAndChildrenIdsById(categoryId);
        const filter = {
            isDeleted: false,
            categoryId: {
                $in: categoryIds,
            },
            createdAt: end
                ? {
                    $gte: new Date(start),
                    $lte: new Date(end),
                }
                : {
                    $gte: new Date('0000'),
                },
            $or: [
                { name: searchTextReg },
                { cn: searchTextReg },
                { tw: searchTextReg },
                { en: searchTextReg },
            ],
        };
        const total = await this.copywritingModel
            .find(filter, {
            createdAt: {
                $dateToString: { format: '%Y-%m-%d', date: '$createdAt' },
            },
        })
            .countDocuments();
        const data = await this.copywritingModel
            .find(filter, { history: 0 })
            .sort({ updatedAt: -1 })
            .skip(Math.max(page - 1, 0) * pageSize)
            .limit(pageSize)
            .populate('categoryId', 'label')
            .lean()
            .then((res) => res.map((each) => {
            var _a, _b;
            return (Object.assign(Object.assign({}, each), { createdAt: (0, utils_1.formatISODate)(each.createdAt), updatedAt: (0, utils_1.formatISODate)(each.updatedAt), categoryId: (_a = each.categoryId) === null || _a === void 0 ? void 0 : _a._id, categoryName: (_b = each.categoryId) === null || _b === void 0 ? void 0 : _b.label }));
        }));
        return { data, total };
    }
    async findOne(id) {
        const res = await this.copywritingModel
            .findOne({
            _id: id,
            isDeleted: false,
        })
            .populate('categoryId', 'label')
            .lean()
            .then((data) => {
            var _a, _b, _c;
            return data
                ? Object.assign(Object.assign({}, data), { categoryId: (_b = (_a = data.categoryId) === null || _a === void 0 ? void 0 : _a._id) === null || _b === void 0 ? void 0 : _b.toString(), categoryName: (_c = data.categoryId) === null || _c === void 0 ? void 0 : _c.label, history: data.history }) : null;
        });
        return res;
    }
    async update(user, id, copywritingDto) {
        let res = null;
        const sameOne = await this.findSameCopywriting(id, copywritingDto.cn, copywritingDto.tw, copywritingDto.en);
        if (sameOne) {
            throw new common_1.HttpException({
                code: response_constant_1.RES_CODE.VALIDATE_ERROR,
                message: '文案内容已存在',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        const target = await this.findOne(id);
        if (target) {
            const newDto = Object.assign(Object.assign({}, target), copywritingDto);
            const type = this.getType(target, newDto);
            let newHistory = target.history;
            if (type) {
                newHistory = [
                    {
                        type,
                        operator: user.name,
                        createdAt: Date.now(),
                        record: newDto,
                    },
                    ...target.history,
                ];
            }
            res = await this.copywritingModel.updateOne({ _id: id }, Object.assign(Object.assign({}, copywritingDto), { history: newHistory }));
        }
        return res;
    }
    async remove(id) {
        const res = await this.copywritingModel.updateOne({ _id: id }, { isDeleted: true });
        return res;
    }
    async batchRemoveByCategoryId(ids) {
        const res = await this.copywritingModel.updateMany({ categoryId: { $in: ids } }, { isDeleted: true });
        return res;
    }
    async findHistory(id) {
        const res = await this.findOne(id);
        return ((res === null || res === void 0 ? void 0 : res.history) || []).map((each) => (Object.assign(Object.assign({}, each), { createdAt: (0, utils_1.formatISODate)(each.createdAt) })));
    }
    getType(oldRecord, newRecord) {
        const resTypes = [];
        if (oldRecord.cn !== newRecord.cn) {
            resTypes.push(constants_1.OPER_TYPE_ENUM.CN);
        }
        if (oldRecord.tw !== newRecord.tw) {
            resTypes.push(constants_1.OPER_TYPE_ENUM.TW);
        }
        if (oldRecord.en !== newRecord.en) {
            resTypes.push(constants_1.OPER_TYPE_ENUM.EN);
        }
        if (oldRecord.name !== newRecord.name ||
            oldRecord.desc !== newRecord.desc ||
            oldRecord.categoryId !== newRecord.categoryId) {
            resTypes.push(constants_1.OPER_TYPE_ENUM.COMMON);
        }
        return resTypes.join(',');
    }
    async findSameCopywriting(id, cn, tw, en) {
        const idFilter = id ? { _id: { $ne: id } } : {};
        const orFilter = [];
        if (cn) {
            orFilter.push({ cn });
        }
        if (tw) {
            orFilter.push({ tw });
        }
        if (en) {
            orFilter.push({ en });
        }
        if (orFilter.length === 0) {
            return null;
        }
        const tgt = await this.copywritingModel.findOne(Object.assign(Object.assign({}, idFilter), { isDeleted: false, $or: orFilter }));
        return tgt;
    }
    async validateContent(key, value, id = '') {
        const idFilter = id ? { _id: { $ne: id } } : {};
        const orFilter = [];
        if (value) {
            orFilter.push({ [key]: value });
        }
        if (Object.keys(idFilter).length === 0 && orFilter.length === 0) {
            return null;
        }
        const tgt = await this.copywritingModel.findOne(Object.assign(Object.assign({}, idFilter), { isDeleted: false, $or: orFilter }));
        return !tgt;
    }
};
exports.CopywritingService = CopywritingService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_1.InjectModel)('Copywriting')),
    __param(1, (0, common_1.Inject)((0, common_1.forwardRef)(() => copywriting_category_service_1.CopywritingCategoryService))),
    __metadata("design:paramtypes", [mongoose_2.Model,
        copywriting_category_service_1.CopywritingCategoryService])
], CopywritingService);
//# sourceMappingURL=copywriting.service.js.map