"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WishGameConfigService = void 0;
const common_1 = require("@nestjs/common");
const mongoose_1 = require("mongoose");
const mongoose_2 = require("@nestjs/mongoose");
const response_constant_1 = require("../../common/constant/response.constant");
const mongo_1 = require("../../common/util/mongo");
const wish_game_service_1 = require("../wish-game/wish-game.service");
const utils_1 = require("../activity-config/common/utils");
const wish_game_config_schema_1 = require("./schemas/wish-game-config.schema");
let WishGameConfigService = exports.WishGameConfigService = class WishGameConfigService {
    constructor(wishGameConfigModel, wishGameService) {
        this.wishGameConfigModel = wishGameConfigModel;
        this.wishGameService = wishGameService;
    }
    async syncData(user, env, port, engineRes) {
        const { date_list } = engineRes;
        const wishGameConfigs = date_list.reduce((r, each) => [
            ...r,
            ...each.wish_game.map((e) => (Object.assign(Object.assign({}, e), { date: each.date }))),
        ], []);
        const valueToIdObj = await this.wishGameService.getIdObjByEnvAndValue(env, wishGameConfigs.map((each) => each.game_name));
        const dates = [];
        for (let i = 0; i < 16; i += 1) {
            dates.push((0, utils_1.getNow)().add(i, 'day').format('YYYY-MM-DD'));
        }
        const dataInDb = await this.wishGameConfigModel.find({
            env,
            port,
            date: {
                $in: dates,
            },
            isDeleted: false,
        });
        const dataInDbObj = {};
        dataInDb.forEach((each) => {
            dataInDbObj[`${each.wishGameId}-${each.date}`] = each._id.toString();
        });
        const opts = [];
        wishGameConfigs.forEach((each) => {
            if (!dataInDbObj[`${valueToIdObj[each.game_name]}-${each.date}`]) {
                opts.push({
                    insertOne: {
                        document: {
                            wishGameId: valueToIdObj[each.game_name],
                            date: each.date,
                            port,
                            creatorId: user.userid,
                            creatorName: user.name,
                            updaterId: user.userid,
                            updaterName: user.name,
                        },
                    },
                });
            }
        });
        const res = await this.wishGameConfigModel.bulkWrite(opts);
        return res;
    }
    async create(user, env, wishGameConfig) {
        const { port, wishGameId, date } = wishGameConfig;
        const sameOne = await this.wishGameConfigModel.findOne({
            env,
            port,
            wishGameId,
            date,
            isDeleted: false,
        });
        if (sameOne) {
            throw new common_1.HttpException({
                code: response_constant_1.RES_CODE.CREATE_ERROR,
                message: '心愿游戏已存在',
            }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        const res = await this.wishGameConfigModel.create(Object.assign(Object.assign({}, wishGameConfig), { env, creatorId: user.userid, creatorName: user.name, updaterId: user.userid, updaterName: user.name }));
        return res;
    }
    async findAllByDays(env, port, dates) {
        const res = await this.wishGameConfigModel.aggregate([
            {
                $match: {
                    env,
                    port,
                    date: { $in: dates },
                    isDeleted: false,
                },
            },
            {
                $lookup: {
                    from: 'wish_games',
                    let: { wishGameId: '$wishGameId' },
                    pipeline: [
                        {
                            $addFields: { _idString: { $toString: '$_id' } },
                        },
                        { $match: { $expr: { $eq: ['$_idString', '$$wishGameId'] } } },
                    ],
                    as: 'wishGame',
                },
            },
            {
                $addFields: {
                    label: { $first: '$wishGame.label' },
                    value: { $first: '$wishGame.value' },
                    createdAt: { $dateToString: (0, mongo_1.default)('$createdAt') },
                    updatedAt: { $dateToString: (0, mongo_1.default)('$updatedAt') },
                },
            },
            { $project: { wishGame: 0 } },
        ]);
        return res;
    }
    async remove(user, id) {
        const res = await this.wishGameConfigModel.findOneAndUpdate({
            _id: id,
        }, {
            isDeleted: true,
            updaterId: user.userid,
            updaterName: user.name,
        });
        return res;
    }
    async clear(user, env, port, date) {
        const res = await this.wishGameConfigModel.updateMany({
            date,
            env,
            port,
            isDeleted: false,
        }, {
            isDeleted: true,
            updaterId: user.userid,
            updaterName: user.name,
        });
        return res;
    }
};
exports.WishGameConfigService = WishGameConfigService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, mongoose_2.InjectModel)(wish_game_config_schema_1.WishGameConfig.name)),
    __metadata("design:paramtypes", [mongoose_1.Model,
        wish_game_service_1.WishGameService])
], WishGameConfigService);
//# sourceMappingURL=wish-game-config.service.js.map