"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WishGameService = void 0;
const mongoose_1 = require("mongoose");
const common_1 = require("@nestjs/common");
const cache_manager_1 = require("@nestjs/cache-manager");
const mongoose_2 = require("@nestjs/mongoose");
const mongo_1 = require("../../common/util/mongo");
const cache_constant_1 = require("../../common/constant/cache.constant");
const activity_engine_service_1 = require("../../common/provider/activity-engine.service");
const response_constant_1 = require("../../common/constant/response.constant");
const wish_game_schema_1 = require("./schemas/wish-game.schema");
let WishGameService = exports.WishGameService = class WishGameService {
    constructor(cacheManager, wishGameModel, activityEngineService) {
        this.cacheManager = cacheManager;
        this.wishGameModel = wishGameModel;
        this.activityEngineService = activityEngineService;
        this.cacheManager = cacheManager;
        this.wishGameModel = wishGameModel;
    }
    getCacheKey(env) {
        const envs = {
            'dev-gtm': cache_constant_1.DEV_GTM_WISH_GAME_LIST,
            'dev-rtm': cache_constant_1.DEV_RTM_WISH_GAME_LIST,
            'prod-rtm': cache_constant_1.PROD_RTM_WISH_GAME_LIST,
            'prod-online': cache_constant_1.PROD_ONLINE_WISH_GAME_LIST,
        };
        return envs[env];
    }
    async sync(user, env) {
        const { wish_themes } = await this.activityEngineService.getDailyConfig();
        const dataInDb = await this.findAll(env);
        const dataInDbObj = {};
        dataInDb.forEach((each) => {
            dataInDbObj[each.value] = each._id;
        });
        const opts = [];
        let order = dataInDb.length;
        wish_themes.forEach((each) => {
            if (dataInDbObj[each.id]) {
                opts.push({
                    updateOne: {
                        filter: { _id: new mongoose_1.Types.ObjectId(dataInDbObj[each.id]) },
                        update: {
                            label: each.name,
                            value: each.id,
                            env,
                            updaterId: user.userid,
                            updaterName: user.name,
                        },
                    },
                });
            }
            else {
                opts.push({
                    insertOne: {
                        document: {
                            label: each.name,
                            value: each.id,
                            env,
                            order: order++,
                            creatorId: user.userid,
                            creatorName: user.name,
                            updaterId: user.userid,
                            updaterName: user.name,
                        },
                    },
                });
            }
        });
        const res = await this.wishGameModel.bulkWrite(opts);
        await this.setAndFindAllInCachFromDb(env);
        return res;
    }
    async create(user, env, wishGameDto) {
        const all = await this.findAll(env);
        const res = await this.wishGameModel.create(Object.assign(Object.assign({}, wishGameDto), { env, order: all.length, creatorId: user.userid, creatorName: user.name, updaterId: user.userid, updaterName: user.name }));
        await this.setAndFindAllInCachFromDb(env);
        return res;
    }
    async setAndFindAllInCachFromDb(env) {
        const res = await this.wishGameModel.aggregate([
            { $match: { isDeleted: false, env } },
            {
                $addFields: {
                    createdAt: { $dateToString: (0, mongo_1.default)('$createdAt') },
                    updatedAt: { $dateToString: (0, mongo_1.default)('$updatedAt') },
                },
            },
            { $sort: { order: 1 } },
        ]);
        await this.cacheManager.set(this.getCacheKey(env), res);
        return res;
    }
    async findAll(env) {
        let res = await this.cacheManager.get(this.getCacheKey(env));
        if (!res) {
            res = await this.setAndFindAllInCachFromDb(env);
        }
        return res;
    }
    async update(user, env, id, wishGameDto) {
        const res = await this.wishGameModel.updateOne({ _id: id }, Object.assign(Object.assign({}, wishGameDto), { updaterId: user.userid, updaterName: user.name }));
        await this.setAndFindAllInCachFromDb(env);
        return res;
    }
    async remove(user, env, id) {
        const res = await this.wishGameModel.updateOne({ _id: id }, {
            isDeleted: true,
            updaterId: user.userid,
            updaterName: user.name,
        });
        await this.setAndFindAllInCachFromDb(env);
        return res;
    }
    async sort(user, env, wishGames) {
        const ops = wishGames.map((each, index) => ({
            updateOne: {
                filter: { _id: each._id },
                update: Object.assign(Object.assign({}, each), { updaterId: user.userid, updaterName: user.name, order: index }),
            },
        }));
        const res = await this.wishGameModel.bulkWrite(ops);
        await this.setAndFindAllInCachFromDb(env);
        return res;
    }
    async getIdObjByEnvAndValue(env, values) {
        const tgtRes = await this.wishGameModel
            .find({
            env: env,
            value: {
                $in: values,
            },
        }, { _id: 1, value: 1 })
            .lean();
        const res = {};
        values.forEach((value) => {
            const tgt = tgtRes.find((each) => each.value === value);
            if (tgt) {
                res[value] = tgt._id.toString();
            }
            else {
                throw new common_1.HttpException({
                    code: response_constant_1.RES_CODE.CREATE_ERROR,
                    message: '同步失败：当前环境和远端环境活动数据不一致',
                }, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
            }
        });
        return res;
    }
};
exports.WishGameService = WishGameService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(cache_manager_1.CACHE_MANAGER)),
    __param(1, (0, mongoose_2.InjectModel)(wish_game_schema_1.WishGame.name)),
    __metadata("design:paramtypes", [Object, mongoose_1.Model,
        activity_engine_service_1.ActivityEngineSerivce])
], WishGameService);
//# sourceMappingURL=wish-game.service.js.map